"use strict";

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

const chalk = require('chalk');

const fs = require('fs');

const fileLoader = require('file-loader');

const {
  getOptions,
  parseQuery
} = require('loader-utils');

const rawLoader = require('raw-loader');

const sharp = require('sharp');

const validateOptions = require('schema-utils');

const fileSchema = require('./config/file.json');

const loaderSchema = require('./config/loader.json');

const format = require('./format');

const lqip = require('./lqip');

const optimize = require('./optimize');

const resize = require('./resize');

const {
  dataURI,
  mergeOptions,
  reportSavings
} = require('./utils'); // Output function


const complete = (output, _ref) => {
  let {
    callback,
    context,
    sizeBefore,
    source
  } = _ref,
      options = _objectWithoutProperties(_ref, ["callback", "context", "sizeBefore", "source"]);

  // Console output
  if (options.optimize.skip || options.emitFile === false) {
    console.log(`${chalk.bold(options.filename)}: skipping…`);
    return callback(null, fileLoader.call(context, output));
  } else if (!options.placeholder) {
    console.log(reportSavings(options, output.byteLength, sizeBefore));
  } // File Output


  if (options.extension === 'svg' && options.inline) {
    // 1. Inline SVGs (SVG code)
    return callback(null, rawLoader.call(context, output.toString()));
  } else if (options.placeholder) {
    // 2. LQIP
    return sharp(source).metadata().then(({
      height,
      width
    }) => callback(null, rawLoader.call(context, lqip(output, _objectSpread({
      width,
      height
    }, options)))));
  } else if (options.inline) {
    // 3. Inlined (base64) images
    return callback(null, rawLoader.call(context, dataURI(output, options.mimetype)));
  } // 4. Optimized images (everything else)


  return callback(null, fileLoader.call(context, output));
};

module.exports = function loader(source) {
  // Let webpack know this loader is async
  const callback = this.async(); // Load file options & break if syntax error

  const fileOptions = this.resourceQuery ? parseQuery(this.resourceQuery) : {};

  if (Object.keys(fileOptions).length) {
    validateOptions(fileSchema, fileOptions, '🚀 Lightspeed Image Loader');
  } // Get global fallback options & break if syntax error


  const loaderOptions = getOptions(this) || {};
  validateOptions(loaderSchema, loaderOptions, '🚀 Lightspeed Image Loader'); // Combine file & fallback options, giving file options priority

  const options = mergeOptions(this.resourcePath, {
    loaderOptions,
    fileOptions
  }); // Save size before optimization

  const sizeBefore = fs.statSync(this.resourcePath).size; // Path 1: skip file

  if (options.optimize.skip || options.emitFile === false) {
    return complete(source, _objectSpread({
      context: this,
      callback,
      source
    }, options));
  } // Path 2: format -> resize -> optimize -> complete


  const srcFormatted = format(_objectSpread({
    context: this
  }, options));
  const srcResized = resize(srcFormatted, options);
  return optimize(srcResized, options).then(srcOptimized => complete(srcOptimized, _objectSpread({
    context: this,
    callback,
    sizeBefore,
    source
  }, options))).catch(error => callback(error));
};

module.exports.raw = true;